<?xml version='1.0' encoding="utf-8"?>
<!DOCTYPE stylesheet [
<!ENTITY newln "&#xA;">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:str="http://exslt.org/strings"
  xmlns:tolatex="http://www.bortzmeyer.org/xmlns/tolatex"
  version='1.0'>
  
  <xsl:import href="common.xsl"/>
  
  <xsl:param name="prefix">https://www.bortzmeyer.org/</xsl:param>
  <xsl:param name="filename">UNSPECIFIED</xsl:param>
  <xsl:param name="add_url" select="true()"/>
  <xsl:param name="add_date" select="true()"/>
  
  <!-- Mostly Latin-1 characters, Unicode support in TeX is too poor -->
  <xsl:variable name="goodletters">0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZàâçèéêëîïôùûüÿÀÂÇÈÉÊËÎÏÔÙÛÜ;:.,_-+°\/%'[](){}""«»~?=!#-+^*$@·|&amp;&lt;&gt;&#x21D0;&#x21D2; &#xa0;&#x0a;&#x0d;&#x09;</xsl:variable>
  
  <tolatex:replacements>
    <!-- http://www-h.eng.cam.ac.uk/help/tpl/textprocessing/teTeX/latex/latex2e-html/ltx-164.html -->
    <tolatex:replacement search="$">DOLLAR-TO-ESCAPE</tolatex:replacement>
    <tolatex:replacement search="\">$\backslash$</tolatex:replacement>
    <!-- Difficult circularity in escaping -->
    <tolatex:replacement search="DOLLAR-TO-ESCAPE">\$</tolatex:replacement>
    <tolatex:replacement search="#">\#</tolatex:replacement>    
    <tolatex:replacement search="_">\_</tolatex:replacement>    
    <tolatex:replacement search="%">\%</tolatex:replacement>    
    <tolatex:replacement search="^">\^{}</tolatex:replacement>    
    <tolatex:replacement search="{">\{</tolatex:replacement>    
    <tolatex:replacement search="}">\}</tolatex:replacement>    
    <tolatex:replacement search="²">$^{2}$</tolatex:replacement>    
    <tolatex:replacement search="&#xa0;">~</tolatex:replacement>
    <tolatex:replacement search="&amp;">\&amp;</tolatex:replacement>
    <tolatex:replacement search="&#x153;">\oe{}</tolatex:replacement>
    <tolatex:replacement search="&#x20ac;">\EUR{}</tolatex:replacement>
    <tolatex:replacement search="&#x00b5;">$\mu$</tolatex:replacement>
    <tolatex:replacement search="…">\ldots</tolatex:replacement>
  </tolatex:replacements>

  <!-- Pour l'élément <computer>, traduit en \texttt{} -->
  <tolatex:verbatimreplacements>
    <tolatex:verbatimreplacement search="$">DOLLAR-TO-ESCAPE</tolatex:verbatimreplacement>
    <tolatex:verbatimreplacement search="\">$\backslash$</tolatex:verbatimreplacement>
    <!-- Difficult circularity in escaping -->
    <tolatex:verbatimreplacement search="DOLLAR-TO-ESCAPE">\$</tolatex:verbatimreplacement>
    <tolatex:verbatimreplacement search="%">\%</tolatex:verbatimreplacement>    
    <tolatex:verbatimreplacement search="²">2</tolatex:verbatimreplacement>    
    <!-- Le suivant ne semble pas nécessaire ? -->
    <tolatex:verbatimreplacement search="'">\'</tolatex:verbatimreplacement>    
    <tolatex:verbatimreplacement search="#">\#</tolatex:verbatimreplacement>    
    <tolatex:verbatimreplacement search="&#x00b5;">micro-</tolatex:verbatimreplacement>
  </tolatex:verbatimreplacements>

  <!-- Pour l'élément <code>, traduit en \begin{info}...\end{info}
       pour lequel il n'y a guère besoin d'échappement.  -->
  <tolatex:hardverbatimreplacements>
    <tolatex:hardverbatimreplacement search="²">2</tolatex:hardverbatimreplacement>    
    <tolatex:hardverbatimreplacement search="&#x00b5;">micro-</tolatex:hardverbatimreplacement>
  </tolatex:hardverbatimreplacements>

  <xsl:variable name="replacements"
    select="document('')/xsl:stylesheet/tolatex:replacements" />

  <xsl:variable name="verbatimreplacements"
    select="document('')/xsl:stylesheet/tolatex:verbatimreplacements" />

  <xsl:variable name="hardverbatimreplacements"
    select="document('')/xsl:stylesheet/tolatex:hardverbatimreplacements" />

  <xsl:template match="p"> 
      <xsl:apply-templates/>
      <xsl:text>&newln;&newln;</xsl:text>
  </xsl:template>
  
  <xsl:template match="enum">
    <xsl:text>\begin{itemize}</xsl:text>
    <xsl:apply-templates/><xsl:text>&newln;</xsl:text>
    <xsl:text>\end{itemize}</xsl:text>
  </xsl:template>
  
  <xsl:template match="item">
    <xsl:text>&newln;</xsl:text><xsl:text>\item </xsl:text><xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="foreign">
    <xsl:text>&quot;\textit{</xsl:text><xsl:apply-templates select="text()|*/text()"/><xsl:text>}&quot;</xsl:text><xsl:apply-templates select="link|wikipedia"/>
  </xsl:template>

  <xsl:template match="image">
     <xsl:if test="not(contains(@name, '.'))">
       <xsl:text>\begin{figure}[ht]\centering\includegraphics{../images/</xsl:text><xsl:value-of select="@name"/><xsl:text>.pdf}\end{figure} </xsl:text>
     <!-- TODO: handle the case where there is an explicit extension -->
     </xsl:if>
  </xsl:template>

  <xsl:template match="unix">
    <xsl:text>\path{</xsl:text><xsl:apply-templates mode="computer"/><xsl:text>}</xsl:text>
  </xsl:template>
  
  <xsl:template match="computer">
    <!-- TODO: <link> URLs in <computer> elements are not printed :-( -->
    <xsl:text>\texttt{</xsl:text><xsl:apply-templates mode="computer"/><xsl:text>}</xsl:text>
  </xsl:template>
  
  <xsl:template match="code">
    <xsl:text>\begin{info}</xsl:text>
    <xsl:apply-templates mode="hardcomputer" select="text()"/>
    <xsl:text>\end{info}&#x0A;</xsl:text>
  </xsl:template>

  <xsl:template match="file">
    <xsl:value-of select="text()"/><xsl:text> (en ligne sur \url{</xsl:text><xsl:value-of select="$prefix"/><xsl:text>files/</xsl:text><xsl:value-of select="@name"/><xsl:text>})</xsl:text>
  </xsl:template>

  <xsl:template match="emphasis">
    <xsl:text>\textbf{</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template match="link">
    <xsl:variable name="href">
      <xsl:choose>
          <xsl:when test="@url">
            <xsl:value-of select="@url"/>
	  </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$prefix"/><xsl:value-of select="@local"/><xsl:text>.html</xsl:text>
	  </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="text() and not (parent::foreign)">
        <xsl:apply-templates select="text()"/><xsl:text> \url{&lt;</xsl:text><xsl:value-of select="$href"/><xsl:text>&gt;}</xsl:text>        
      </xsl:when>
      <xsl:otherwise>
        <xsl:if test="text() and parent::foreign">
          <xsl:text> </xsl:text>
        </xsl:if>
        <xsl:text>\url{&lt;</xsl:text><xsl:value-of select="$href"/><xsl:text>&gt;}</xsl:text>
        <xsl:if test="text() and parent::foreign">
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="wikipedia">
    <!-- Ignore Wikipedia -->
    <xsl:if test="text() and not (parent::foreign)">
       <xsl:apply-templates/>
    </xsl:if>
  </xsl:template>

  <xsl:template match="rfc">
    <xsl:variable name="num">
      <xsl:value-of select="@num"/>
    </xsl:variable>
    <xsl:text>\rfc{</xsl:text><xsl:value-of select="@num"/><xsl:text>}</xsl:text>
  </xsl:template>

  <xsl:template name="url_header">
    <xsl:if test="$add_url = 'true'">
      <xsl:text>\url{</xsl:text><xsl:value-of select="$prefix"/><xsl:value-of select="$filename"/><xsl:text>.html}</xsl:text>
      <xsl:text>&newln;</xsl:text>
      <xsl:text>&newln;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template name="end_header">
    <xsl:text>----------------------------</xsl:text>
    <xsl:text>&newln;</xsl:text>
    <xsl:text>&newln;</xsl:text>
  </xsl:template>

  <xsl:template name="preamble">
    <xsl:param name="title_prefix"/>
    <xsl:param name="extra_preamble"/>
    <xsl:text>
\documentclass[a4paper,twoside]{article}
\usepackage[utf8x]{inputenc}
\usepackage[french]{babel}
\usepackage{url}
\usepackage{verbatim}
\usepackage{eurosym}
\usepackage{ifthen}
\usepackage{vmargin}
\usepackage{fancyhdr}
\usepackage{palatino}
\usepackage{underscore}
\usepackage{graphicx}

\setlength{\parskip}{20pt plus 10pt minus 10pt} 
% More space before the footnotes
\addtolength{\skip\footins}{0.7 cm}

\pagestyle{fancy}
      \fancyhead[C]{\thepage}
      \fancyfoot[C]{----------------------------\\
               \texttt{</xsl:text><xsl:value-of select="$prefix"/><xsl:value-of select="$filename"/>.html<xsl:text>}}

% Options and their defaults
\newboolean{rfcfootnote}
\setboolean{rfcfootnote}{true}
\DeclareOption{norfcfootnote}{\setboolean{rfcfootnote}{false}}

\ProcessOptions

% The first time we mention a RFC, add a reference 
\newcounter{rfcfirst}
\newcommand{\rfc}[1]{RFC #1\ifthenelse{\boolean{rfcfootnote}}%
{\ifthenelse{\value{rfcfirst} = 0}%
   {\footnote{Pour voir le RFC de numéro NNN,
   \url{https://www.ietf.org/rfc/rfcNNN.txt}, par exemple
   \url{https://www.ietf.org/rfc/rfc#1.txt}}\stepcounter{rfcfirst}}%
   {}%
}%
{}%	
}

\newcounter{unicodewarningfirst}
\newcommand{\unicodewarning}[0]{
\ifthenelse{\value{unicodewarningfirst} = 0}
{
\footnote{Car trop difficile à faire afficher par \LaTeX}
\stepcounter{unicodewarningfirst}
}
{}
}

\newenvironment{info}
        {\footnotesize\verbatim}
        {\endverbatim}
\newcommand{\dns}[1]{``\url{#1}''}

\author{Stéphane Bortzmeyer\\
\url{&lt;stephane+blog@bortzmeyer.org&gt;}}
    \title{</xsl:text>
    <xsl:variable name="full_title">
      <xsl:value-of select="$title_prefix"/>
      <xsl:value-of select="string(@title)"/>
    </xsl:variable>
    <xsl:call-template name="escape_text_attr">
       <xsl:with-param name="data">
         <xsl:call-template name="replace">
           <xsl:with-param name="content" select="$full_title"/>
         </xsl:call-template>
       </xsl:with-param>
    </xsl:call-template>
    <xsl:text>}
  </xsl:text><xsl:if test="$add_date = 'true'">
      <xsl:text>\date{</xsl:text>
      <xsl:apply-templates select="date"/>
      <xsl:if test="update_date">
        <xsl:text>. </xsl:text>
        <xsl:apply-templates select="update_date"/>
      </xsl:if>
      <xsl:text>}</xsl:text>
      <xsl:text>&newln;</xsl:text>
      <xsl:text>&newln;</xsl:text>
    </xsl:if>
    <xsl:text>
\setmarginsrb{2.1cm}{2cm}{2.9cm}{3cm}{0.5cm}{0.5cm}{0.5cm}{0.5cm}
\begin{document}
\maketitle
    </xsl:text>    
    <xsl:value-of select="$extra_preamble"/>
    <xsl:text>&newln;</xsl:text>
  </xsl:template>

  <xsl:template name="postamble">
    <xsl:text>
\end{document}
    </xsl:text>    
  </xsl:template>

  <xsl:template name="string_replace">
    <xsl:param name="string"/>
    <xsl:param name="search"/>
    <xsl:param name="replace"/>
    <xsl:choose>
      <xsl:when test="contains($string, $search)">
        <xsl:variable name="rest">
          <xsl:call-template name="string_replace">
            <xsl:with-param name="string" select="substring-after
                                                  ($string, $search)"/>
            <xsl:with-param name="search" select="$search"/>
            <xsl:with-param name="replace" select="$replace"/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select="concat(substring-before($string, $search), $replace, $rest)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$string"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="replace">
    <xsl:param name="position" select="1"/>
    <xsl:param name="content"/>
    <xsl:variable name="transformed">
    <xsl:call-template name="string_replace">
      <xsl:with-param name="string" select="$content" />
      <xsl:with-param name="search"
        select="$replacements/tolatex:replacement[$position]/@search" /> 
      <xsl:with-param name="replace"
        select="$replacements/tolatex:replacement[$position]" /> 
    </xsl:call-template>
          </xsl:variable>
          <xsl:choose>
    <xsl:when test="$position &lt; count($replacements/tolatex:replacement)">
      <xsl:call-template name="replace">
        <xsl:with-param name="position" select="$position + 1"/>
        <xsl:with-param name="content" select="$transformed"/>        
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
    <xsl:value-of select="$transformed"/>
    </xsl:otherwise>
  </xsl:choose>
  </xsl:template>

  <xsl:template name="verbatimreplace">
    <xsl:param name="position" select="1"/>
    <xsl:param name="content"/>
    <xsl:variable name="transformed">
    <xsl:call-template name="string_replace">
      <xsl:with-param name="string" select="$content" />
      <xsl:with-param name="search"
        select="$verbatimreplacements/tolatex:verbatimreplacement[$position]/@search" /> 
      <xsl:with-param name="replace"
        select="$verbatimreplacements/tolatex:verbatimreplacement[$position]" /> 
    </xsl:call-template>
          </xsl:variable>
          <xsl:choose>
    <xsl:when test="$position &lt; count($verbatimreplacements/tolatex:verbatimreplacement)">
      <xsl:call-template name="verbatimreplace">
        <xsl:with-param name="position" select="$position + 1"/>
        <xsl:with-param name="content" select="$transformed"/>        
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
    <xsl:value-of select="$transformed"/>
    </xsl:otherwise>
  </xsl:choose>
  </xsl:template>

  <!-- TODO: a bit of refactoring, merge this template with the
       previous one, with parametrization. -->
  <xsl:template name="hardverbatimreplace">
    <xsl:param name="position" select="1"/>
    <xsl:param name="content"/>
    <xsl:variable name="transformed">
    <xsl:call-template name="string_replace">
      <xsl:with-param name="string" select="$content" />
      <xsl:with-param name="search"
        select="$hardverbatimreplacements/tolatex:hardverbatimreplacement[$position]/@search" /> 
      <xsl:with-param name="replace"
        select="$hardverbatimreplacements/tolatex:hardverbatimreplacement[$position]" /> 
    </xsl:call-template>
          </xsl:variable>
          <xsl:choose>
    <xsl:when test="$position &lt; count($hardverbatimreplacements/tolatex:hardverbatimreplacement)">
      <xsl:call-template name="hardverbatimreplace">
        <xsl:with-param name="position" select="$position + 1"/>
        <xsl:with-param name="content" select="$transformed"/>        
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
    <xsl:value-of select="$transformed"/>
    </xsl:otherwise>
  </xsl:choose>
  </xsl:template>

  <xsl:template mode="computer" match="text()">
    <xsl:variable name="position" select="1"/>
    <xsl:variable name="data">
      <xsl:call-template name="verbatimreplace">
        <xsl:with-param name="position" select="$position"/>
        <xsl:with-param name="content" select="string(.)"/>
      </xsl:call-template>
    </xsl:variable>
     <xsl:call-template name="escape_text">
         <xsl:with-param name="data">
          <xsl:value-of select='$data'/>"/>
	 </xsl:with-param>
    </xsl:call-template>
  </xsl:template>

  <xsl:template mode="hardcomputer" match="text()">
    <xsl:variable name="position" select="1"/>
    <xsl:variable name="data">
      <xsl:call-template name="hardverbatimreplace">
        <xsl:with-param name="position" select="$position"/>
        <xsl:with-param name="content" select="string(.)"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$data"/>
  </xsl:template>

  <xsl:template name="escape_text" match="text()">
    <xsl:variable name="position" select="1"/>
    <xsl:variable name="data">
      <xsl:call-template name="replace">
        <xsl:with-param name="position" select="$position"/>
        <xsl:with-param name="content" select="string(.)"/>
      </xsl:call-template>
    </xsl:variable>
    <xsl:for-each select="str:tokenize($data, '')">
      <xsl:choose>
        <xsl:when test="string-length(translate(text(), $goodletters, '')) != 0">[Caractère Unicode non montré\unicodewarning]</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
      </xsl:for-each>
  </xsl:template>

  <xsl:template name="escape_text_attr">
    <xsl:param name="data"/>
    <xsl:for-each select="str:tokenize($data, '')">
      <xsl:choose>
        <xsl:when test="string-length(translate(text(), $goodletters, '')) != 0">[Caractère Unicode non montré\unicodewarning]</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="."/>
        </xsl:otherwise>
      </xsl:choose>
      </xsl:for-each>
  </xsl:template>

</xsl:stylesheet>
