<?xml version='1.0' encoding="iso-8859-1"?>
<!DOCTYPE stylesheet [
<!ENTITY newln "&#xA;">
<!ENTITY nextln "&#x85;">
<!ENTITY nobreak "&#x83;">
]>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:str="http://exslt.org/strings"
  version='1.0'>

  <xsl:import href="common.xsl"/>
  <xsl:import href="text-format.xsl"/>

  <xsl:param name="prefix">https://www.bortzmeyer.org/</xsl:param>
  <xsl:param name="filename">UNSPECIFIED</xsl:param>
  <xsl:param name="add_url" select="true()"/>
  <xsl:param name="add_date" select="true()"/>

  <xsl:template match="p"> 
    <xsl:variable name="all">
      <xsl:apply-templates/>
    </xsl:variable>
    <!-- &nextln; has been added by the templates to indicate the start
         of a chunk (each chunk is formatted independently) -->
    <xsl:for-each select="str:split($all, '&nextln;')">
      <xsl:variable name="current">
        <xsl:value-of select="."/>
      </xsl:variable>
      <xsl:choose>
        <!-- In addition, if a chunk starts with &nobreak;, it is *not*
             formatted at all. -->
        <xsl:when test="substring($current,1,1) = '&nobreak;'">
          <xsl:text>&newln;</xsl:text><xsl:value-of select="substring($current, 3, string-length($current))" /><xsl:text>&newln;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="format">
            <xsl:with-param name="txt">
              <xsl:value-of select="normalize-space(.)"/>
            </xsl:with-param>
            <xsl:with-param name="width" select="72"/>
            <xsl:with-param name="space-padding">0</xsl:with-param>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="enum">
    <xsl:apply-templates/><xsl:text>&nextln;</xsl:text>
  </xsl:template>
  
  <xsl:template match="item">
    <xsl:text>&nextln;</xsl:text><xsl:text>* </xsl:text><xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="foreign">
    <xsl:text>&quot;</xsl:text><xsl:apply-templates/><xsl:text>&quot;</xsl:text>
  </xsl:template>
  
 <xsl:template match="image">
    Image (<xsl:value-of select="text()"/>, <xsl:value-of select="$prefix"/>images/<xsl:value-of select="@name"/>)
  </xsl:template>

  <xsl:template match="computer">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="p/code">
    <!-- Add &nobreak; to indicate that it must not be formatted --> 
    <xsl:text>&nextln;</xsl:text><xsl:text>&nobreak;</xsl:text>
    <xsl:value-of select="text()"/>
  </xsl:template>
  
  <xsl:template match="code">
    <xsl:value-of select="text()"/>
  </xsl:template>

  <xsl:template match="file">
    <xsl:value-of select="text()"/><xsl:text> (en ligne sur </xsl:text><xsl:value-of select="$prefix"/><xsl:text>files/</xsl:text><xsl:value-of select="@name"/><xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template match="emphasis">
    <xsl:text>*</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>*</xsl:text>
  </xsl:template>

  <xsl:template match="link">
    <xsl:variable name="href">
      <xsl:choose>
          <xsl:when test="@url">
            <xsl:value-of select="@url"/>
	  </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$prefix"/><xsl:value-of select="@local"/><xsl:text>.html</xsl:text>
	  </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="text()">
    <xsl:value-of select="text()"/><xsl:text> &lt;</xsl:text><xsl:value-of select="$href"/><xsl:text>&gt;</xsl:text>        
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$href"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="wikipedia">
    <!-- Ignore Wikipedia -->
    <xsl:value-of select="text()"/>
  </xsl:template>

  <xsl:template match="rfc">
    <xsl:variable name="num">
      <xsl:value-of select="@num"/>
    </xsl:variable>
    <xsl:text>RFC </xsl:text><xsl:value-of select="@num"/>
  </xsl:template>

  <xsl:template name="date_header">
    <xsl:if test="$add_date = 'true'">
      <xsl:apply-templates select="date"/>
      <xsl:if test="update_date">
        <xsl:text>. </xsl:text>
        <xsl:apply-templates select="update_date"/>
      </xsl:if>
      <xsl:text>&#10;</xsl:text>
      <xsl:text>&#10;</xsl:text>
    </xsl:if>
  </xsl:template>  

  <xsl:template match="text()">
    <xsl:value-of select="."/>
  </xsl:template>

  <xsl:template name="url_header">
    <xsl:if test="$add_url = 'true'">
      <xsl:value-of select="$prefix"/><xsl:value-of select="$filename"/><xsl:text>.html</xsl:text>
      <xsl:text>&#10;</xsl:text>
      <xsl:text>&#10;</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template name="end_header">
    <xsl:text>----------------------------</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="html">
    <xsl:text>[Code HTML omis]</xsl:text>
  </xsl:template>

</xsl:stylesheet>
