<?xml version="1.0"?>
<xsl:stylesheet xsl:version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"    
  version="1.0"             
  xmlns:date="http://exslt.org/dates-and-times"
  xmlns:str="http://exslt.org/strings"
  exclude-result-prefixes="str date">

  <xsl:output method="xml" encoding="UTF-8"/>

  <xsl:param name="num"/>
  <xsl:param name="canon_num">
    <!-- Pad it to four digits -->
    <xsl:call-template name="prepend-pad">
      <xsl:with-param name="padChar">0</xsl:with-param>
      <xsl:with-param name="padVar"><xsl:value-of select="$num"/></xsl:with-param>
      <xsl:with-param name="length">4</xsl:with-param>
    </xsl:call-template>
  </xsl:param>
  <xsl:param name="outside_date">TODO</xsl:param>
  <xsl:param name="current_date">
    <xsl:choose>
      <!-- str:align is mandatory because date:month-in-year can
           return one-digit numbers (1 for January) which are not accepted as
           xsd:dateTime. I do not find a way to convince date:month-in-year to 
           yield padded two-digits numbers. -->
      <xsl:when test="function-available('date:date-time') and function-available('str:align')">
        <xsl:variable name="now" select="date:date-time()"/>
        <xsl:value-of select="date:year($now)"/>
        <xsl:text>-</xsl:text>
        <xsl:value-of select="str:align(date:month-in-year($now), '00', 'right')"/>
        <xsl:text>-</xsl:text>
        <xsl:value-of select="str:align(date:day-in-month($now), '00', 'right')"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$outside_date"/>
      </xsl:otherwise>
    </xsl:choose>    
  </xsl:param>

  <!-- Found on http://www.dpawson.co.uk/xsl/sect2/padding.html
       Written by Nate Austin -->
<xsl:template name="prepend-pad">    
  <!-- recursive template to right justify and prepend-->
  <!-- the value with whatever padChar is passed in   -->
    <xsl:param name="padChar"> </xsl:param>
    <xsl:param name="padVar"/>
    <xsl:param name="length"/>
    <xsl:choose>
      <xsl:when test="string-length($padVar) &lt; $length">
        <xsl:call-template name="prepend-pad">
          <xsl:with-param name="padChar" select="$padChar"/>
          <xsl:with-param name="padVar" select="concat($padChar,$padVar)"/>
          <xsl:with-param name="length" select="$length"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of 
	  select="substring($padVar,string-length($padVar) -
	  $length + 1)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="append-pad">    
  <!-- recursive template to left justify and append  -->
  <!-- the value with whatever padChar is passed in   -->
    <xsl:param name="padChar"> </xsl:param>
    <xsl:param name="padVar"/>
    <xsl:param name="length"/>
    <xsl:choose>
      <xsl:when test="string-length($padVar) &lt; $length">
        <xsl:call-template name="append-pad">
          <xsl:with-param name="padChar" select="$padChar"/>
          <xsl:with-param name="padVar" select="concat($padVar,$padChar)"/>
          <xsl:with-param name="length" select="$length"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="substring($padVar,1,$length)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="/">
    <!-- <xsl:comment>Generated for RFC <xsl:value-of select="$canon_num"/></xsl:comment> -->
    <xsl:apply-templates select="rfc/back/references/reference"/> 
  </xsl:template>

  <!-- TODO: RFC before the 1000 -->
  <xsl:template match="reference[@anchor]">
    <xsl:if test="substring-after(@anchor,'RFC') = $canon_num">
      <xsl:variable name="id"><xsl:value-of select="substring-after(@anchor,'RFC')"/></xsl:variable>
      <rfcdesc>
        <xsl:attribute name="num"><xsl:value-of select="$id"/></xsl:attribute>
        <xsl:attribute name="title">
          <xsl:value-of select="front/title"/>
        </xsl:attribute>
        <xsl:attribute name="status"><xsl:text>TODO</xsl:text></xsl:attribute>
        <xsl:text>&#10;</xsl:text>
        <authors>
          <xsl:apply-templates select="front/author"/>
        </authors>
        <xsl:text>&#10;</xsl:text>
        <xsl:apply-templates select="front/date"/>
        <xsl:text>&#10;</xsl:text>
        <date>TODO</date> <!-- We could use <xsl:value-of
	select="$current_date"/> but articles are written long after,
	so it may be better to delay the indication of the date -->
        <xsl:text>&#10;</xsl:text>
        <content>
          <xsl:text>&#10;</xsl:text>
          <p>TODO: relecture</p>
          <p>TODO: liens WP</p>
          <xsl:text>&#10;</xsl:text>
          <p>TODO</p>
          <xsl:text>&#10;</xsl:text>
        </content>
        <xsl:text>&#10;</xsl:text>
      </rfcdesc>      
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="date">
    <rfcdate><month><xsl:value-of select="@month"/></month><year><xsl:value-of select="@year"/></year></rfcdate>
  </xsl:template>

  <xsl:template match="author">
    <author><xsl:value-of select="@fullname"/><xsl:if test="organization/text()"><xsl:text> (</xsl:text><xsl:value-of select="organization/text()"/><xsl:text>)</xsl:text></xsl:if></author>
  </xsl:template>

  <xsl:template match="*"/> <!-- Ignore the rest -->

</xsl:stylesheet>

