#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

import os
import sys
import re
import locale
import io

from simpletal import simpleTAL, simpleTALES, simpleTALUtils
import psycopg2

import statutils

encoding = "utf-8"
statutils.encoding = encoding
base_uses_blog = False
general_tmpl_name = "general.tal_html"
tmpl_name =  "largest-smallest.tal_html"
gal_title = u"Plus grands et plus petits articles de ce blog"
top = 30
bottom = top

locale.setlocale(locale.LC_NUMERIC, 'fr_FR')

html_page = None
for directory in [os.environ['HOME'] + "/Blog/auto/",
                  "/var/www/www.bortzmeyer.org/auto/",
                  "/var/www/wsgis-bortzmeyer/auto/"]:
    if os.path.isfile(directory + tmpl_name):
        html_page = open(directory + tmpl_name)
        general_html_page = open(directory  + general_tmpl_name)
        break
if html_page is None:
    raise Exception("%s not found anywhere" % tmpl_name)
template = simpleTAL.compileXMLTemplate(html_page)
general_template = simpleTAL.compileXMLTemplate(general_html_page)
context = simpleTALES.Context()
general_context = simpleTALES.Context()
rendered = io.StringIO()
gal_rendered = io.StringIO()
 
conn = psycopg2.connect("dbname=blog")
cursor = conn.cursor()
cursor.execute("SET client_encoding TO '%s'" % encoding)

cursor.execute("SELECT id, Blog.num_characters(id), filename, title FROM Blog.Articles ORDER BY Blog.num_characters(id) DESC;")
results = []
num = 0
for tuple in cursor.fetchall():
    num += 1
    if num > top:
        break
    chars = int(tuple[1])
    filename = tuple[2]
    title = tuple[3]
    results.append({'count': chars,
                    'url': "/" + re.sub("\.([a-z]+_)?xml$", ".html", filename),
                    'title': title})
context.addGlobal("largest_articles_chars", results)

cursor.execute("SELECT id, Blog.num_characters(id), filename, title FROM Blog.Articles ORDER BY Blog.num_characters(id);")
results = []
num = 0
for tuple in cursor.fetchall():
    num += 1
    if num > bottom:
        break
    chars = int(tuple[1])
    filename = tuple[2]
    title = tuple[3]
    results.append({'count': chars,
                    'url': "/" + re.sub("\.([a-z]+_)?xml$", ".html", filename),
                    'title': title})
context.addGlobal("smallest_articles_chars", results)

cursor.execute("SELECT id, Blog.num_words(id), filename, title FROM Blog.Articles ORDER BY Blog.num_words(id) DESC;")
results = []
num = 0
for tuple in cursor.fetchall():
    num += 1
    if num > top:
        break
    chars = int(tuple[1])
    filename = tuple[2]
    title = tuple[3]
    results.append({'count': chars,
                    'url': "/" + re.sub("\.([a-z]+_)?xml$", ".html", filename),
                    'title': title})
context.addGlobal("largest_articles_words", results)

cursor.execute("SELECT id, Blog.num_words(id), filename, title FROM Blog.Articles ORDER BY Blog.num_words(id);")
results = []
num = 0
for tuple in cursor.fetchall():
    num += 1
    if num > bottom:
        break
    chars = int(tuple[1])
    filename = tuple[2]
    title = tuple[3]
    results.append({'count': chars,
                    'url': "/" + re.sub("\.([a-z]+_)?xml$", ".html", filename),
                    'title': title})
context.addGlobal("smallest_articles_words", results)

cursor.close()
conn.close()

general_context.addGlobal("time", statutils.current_datetime())
template.expand (context, rendered, outputEncoding=encoding, suppressXMLDeclaration=True)
general_context.addGlobal("content", rendered.getvalue())
general_context.addGlobal("title", gal_title)
general_template.expand (general_context, gal_rendered, outputEncoding=encoding,
                         docType=statutils.doctype)
    
print(gal_rendered.getvalue())  
