#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

import os
import sys
import re
import locale
import io

from simpletal import simpleTAL, simpleTALES, simpleTALUtils
import psycopg2

import statutils

encoding = "utf-8"
statutils.encoding = encoding
base_uses_blog = False
MAX_LENGTH = 10
general_tmpl_name = "general.tal_html"
tmpl_name =  "wikipedia-usage.tal_html"
title = u"Références vers Wikipédia"

html_page = None
for directory in [os.environ['HOME'] + "/Blog/auto/",
                  "/var/www/www.bortzmeyer.org/auto/",
                  "/var/www/wsgis-bortzmeyer/auto/"]:
    if os.path.isfile(directory + tmpl_name):
        html_page = open(directory + tmpl_name)
        general_html_page = open(directory  + general_tmpl_name)
        break
if html_page is None:
    raise Exception("%s not found anywhere" % tmpl_name)
template = simpleTAL.compileXMLTemplate(html_page)
general_template = simpleTAL.compileXMLTemplate(general_html_page)
context = simpleTALES.Context()
general_context = simpleTALES.Context()
rendered = io.StringIO()
gal_rendered = io.StringIO()
 
conn = psycopg2.connect("dbname=blog")
cursor = conn.cursor()
cursor.execute("SET client_encoding TO '%s'" % encoding)
cursor.execute("SELECT count(id) FROM Blog.Wikipedia;")
context.addGlobal("num_wp", int(cursor.fetchone()[0]))
results = []
int_results = {}
names = []
cursor.execute("SELECT title, filename, name, lang, Blog.num_refs(wikipedia) FROM Blog.Wikipedia, Blog.Wikipedia_Articles, Blog.Articles WHERE wikipedia = Blog.Wikipedia.id AND article = Blog.Articles.id ORDER BY Blog.num_refs(wikipedia) DESC, name;")
for result in cursor.fetchall():
    name = result[2]
    lang = result[3]
    uname = name + '-' + lang
    linkarticle = '/' + re.sub("\.xml$", ".html", result[1])
    if uname not in int_results:
        names.append(uname)
        int_results[uname] = {'name': name,
                              'lang': lang,
                             'count': result[4],
                             'overflow': False,
                             'url': "http://%s.wikipedia.org/wiki/%s" % (lang, name),
                            'articles': [{'filename': linkarticle,
                                         'title': result[0]}, ]}
    else:
        if len(int_results[uname]['articles']) > MAX_LENGTH:
            int_results[uname]['overflow'] = True
        else:
            int_results[uname]['articles'].append({'filename': linkarticle,
                                             'title': result[0]})
        
for uname in names:
    results.append({'name': int_results[uname]['name'],
                    'url': int_results[uname]['url'],
                    'lang': int_results[uname]['lang'],
                    'overflow': int_results[uname]['overflow'],
                    'count': int_results[uname]['count'],
                    'articles': int_results[uname]['articles']})
    
context.addGlobal("wp", results)
cursor.close()
conn.close()

general_context.addGlobal("time", statutils.current_datetime())
template.expand (context, rendered, outputEncoding=encoding, suppressXMLDeclaration=True)
general_context.addGlobal("content", rendered.getvalue())
general_context.addGlobal("title", title)
general_template.expand (general_context, gal_rendered, outputEncoding=encoding,
                         docType=statutils.doctype)
    
print(gal_rendered.getvalue())  
