#!/usr/bin/env python3

import os
import sys
import re
import cgi
import string
import urllib.request, urllib.parse, urllib.error
import time

import psycopg2

encoding = "utf-8"
base_uses_blog = False
start_url = "https://www.bortzmeyer.org"

def application(environ, start_response):
    status = '200 OK'
    response_headers = []
    path = environ.get('PATH_INFO', '')
    # Configured with WSGI, PATH_INFO is just the NI, omitting the "/.well-known/ni" prefix
    (empty, algorithm, digest) = path.split('/', 3)
    if algorithm != "sha256":
        status = '401 Wrong algorithm'
        output = "Unknow algorithm %s\n" % algorithm
        response_headers.append(('Content-Length', str(len(output.encode))))
        response_headers.append(('Content-type', 'text/plain'))
        start_response(status, response_headers)
        return [output.encode()]
    cursor.execute("SELECT filename FROM Blog.articles WHERE hash = %(hash)s", {'hash': digest + '='})
    if cursor.rowcount == 0:
        status = '404 Not found'
        output = "No file for this hash\n"
        response_headers.append(('Content-Length', str(len(output.encode()))))
        response_headers.append(('Content-type', 'text/plain'))
        start_response(status, response_headers)
        return [output.encode()]
    filename = cursor.fetchone()[0]
    # We return the XML instead of the HTML because it is the one
    # stable, and secured by the hash.
    url = "%s/%s" % (start_url, filename)
    status = '307 Redirect' # Or 302? The difference in the RFC is not obvious
    # TODO: RFC 2616 says to send a short HTML blurb with a link,
    # instead. I hesitate.
    output = b''
    response_headers.append(('Location', url))
    # TODO: an Expires field, set as one hour later (we do not index in real-time)
    # TODO: a Server field? Or add something to the one generated by Apache
    start_response(status, response_headers)
    return [output]

# Initialization
conn = psycopg2.connect("dbname=blog")
cursor = conn.cursor()

program = "ni.py" # When running under mod_wsgi, sys.argv does not contain my name

if __name__ == '__main__':
    program = os.path.basename(sys.argv[0])
    base_uses_blog = True
    def local_debug(status, headers):
        print(status)
        print(headers)
    print("".join(application(os.environ, local_debug)))
    
